﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using CalculationWorks.FreeEdition.BusinessModel;
using CalculationWorks.FreeEdition.BusinessModel.Behaviors;

namespace BcfSampleConsole {
    class Program {
        static void Main(string[] args) {
            Console.WriteLine("Start");

            var modelSetup = new SampleModelSetup();
            modelSetup.BehaviorItems.Add(new BcfDataSetDiagnostics() { Enabled = true });
            modelSetup.BehaviorItems.Add(new BcfDataSetUndoRepository() { });

            var model = new SampleModel(modelSetup);
            model.Behavior.Diagnostics.EndCalculatingCell += Diagnostics_EndCalculatingCell;
            
            Console.WriteLine("Adding Causeway without transaction");
            var causeway1 = AddCauseway(model, 3);
            Console.WriteLine("Done");
            WriteCauseway(causeway1);
            Console.WriteLine();

            Console.WriteLine("Adding Causeway with transaction");
            var transaction = model.BeginTransaction();
            var causeway2 = AddCauseway(model, 3);
            transaction.Commit();
            Console.WriteLine("Done");
            WriteCauseway(causeway2);
            Console.WriteLine();

            model.EnableChangeTracking();
            Console.WriteLine("Changing Causeway width");
            causeway1.Width = 2.3M;
            Console.WriteLine("Done");
            WriteCauseway(causeway1);
            Console.WriteLine();

            Console.WriteLine("Undoing Causeway width change");
            model.Undo();
            Console.WriteLine("Done");
            WriteCauseway(causeway1);
            Console.WriteLine();

            Console.ReadKey();
        }

        static void Diagnostics_EndCalculatingCell(object sender, CalculationWorks.FreeEdition.BusinessModel.Behaviors.Diagnostics.CalculatingCellEventArgs e) {
            Console.WriteLine(
                string.Format("Calculated Table:{0} Column:{1} Row:{2}",
                e.Cell.Column.Table.Name,
                e.Cell.Column.Name,
                e.Cell.Row.GetTable().Rows.IndexOf(e.Cell.Row)
            ));
        }

        static CausewayRow AddCauseway(SampleModel model, int segmentCount) {
            var causeway = model.CausewayTable.CreateNewRow();
            causeway.Width = 1.5M;
            causeway.AddRowToTable();

            for (int i = 0; i < segmentCount; i++) {
                var segment = model.SegmentTable.CreateNewRow();
                segment.Length = i + 10;
                segment.AddRowToTable();
                segment.CausewayRow = causeway;
            }
            return causeway;
        }

        static void WriteCauseway(CausewayRow causeway) {
            Console.WriteLine(string.Format("causeway width: {0} size: {1}", causeway.Width, causeway.Size));
            foreach (var segment in causeway.GetCausewaySegmentRows()) {
                Console.WriteLine(string.Format("segment length: {0} size: {1}", segment.Length, segment.Size));
            }
        }
    }
}
